/*
 * Copyright 2022 The Android Open Source Project
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       https://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

import com.android.build.api.variant.LibraryAndroidComponentsExtension
import com.android.build.gradle.LibraryExtension
import com.mrl.pixiv.buildsrc.androidTestImplementation
import com.mrl.pixiv.buildsrc.configureKotlinAndroid
import com.mrl.pixiv.buildsrc.disableUnnecessaryAndroidTests
import com.mrl.pixiv.buildsrc.implementation
import com.mrl.pixiv.buildsrc.ksp
import com.mrl.pixiv.buildsrc.testImplementation
import org.gradle.api.Plugin
import org.gradle.api.Project
import org.gradle.api.artifacts.VersionCatalogsExtension
import org.gradle.kotlin.dsl.configure
import org.gradle.kotlin.dsl.dependencies
import org.gradle.kotlin.dsl.getByType
import org.gradle.kotlin.dsl.kotlin


class AndroidLibraryConventionPlugin : Plugin<Project> {
    override fun apply(target: Project) {
        with(target) {
            with(pluginManager) {
                apply("com.android.library")
                apply("org.jetbrains.kotlin.android")
                apply("com.google.devtools.ksp")
            }

            extensions.configure<LibraryExtension> {
                configureKotlinAndroid(this)
                defaultConfig.targetSdk = 35

                flavorDimensions += "version"
                productFlavors {
                    create("default") {
                        dimension = flavorDimensionList[0]
                    }
                    create("foss") {
                        dimension = flavorDimensionList[0]
                    }
                }
            }
            extensions.configure<LibraryAndroidComponentsExtension> {
                disableUnnecessaryAndroidTests(target)
            }
            val libs = extensions.getByType<VersionCatalogsExtension>().named("libs")
            val androidx = extensions.getByType<VersionCatalogsExtension>().named("androidx")
            configurations.configureEach {
                resolutionStrategy {
                    force(libs.findLibrary("junit-junit").get())
                }
            }
            dependencies {
                implementation(androidx.findLibrary("core-ktx").get())
                implementation(androidx.findLibrary("appcompat").get())
                ksp(libs.findLibrary("koin.ksp.compiler").get())

                androidTestImplementation(kotlin("test"))
                testImplementation(kotlin("test"))
            }
        }
    }
}